/*
    Copyright 2010 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#else
    #define stricmp strcasecmp
    #define stristr strcasestr
#endif

typedef unsigned char   u8;



#define VER     "0.1"



void thebatpwd(u8 *pwd) {
    static const u8 table[] =
        "\xff\x41\x41\x41\x41\x41\x41\x41\x41\x41\xfe\x41\x41\xfe\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x00\x41\x01\x41\x41"
        "\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x41\x41\x41\x41\x41\x41"
        "\x41\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a"
        "\x1b\x1c\x1d\x1e\x1f\x20\x21\x22\x23\x24\x25\x41\x41\x41\x41\x41"
        "\x41\x26\x27\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f\x30\x31\x32\x33\x34"
        "\x35\x36\x37\x38\x39\x3a\x3b\x3c\x3d\x3e\x3f\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41"
        "\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41\x41";
    u8      a,
            d,
            *p,
            *o,
            *l;

    // the following is a modified base64 algorithm
#define THEBATPWD_GET(X) \
    do { \
        X = table[*p++]; \
        if(X == 0xff) goto next_step; \
    } while(X == 0xfe); \
    if(X > 'A') goto next_step;

    o = pwd;
    p = pwd;
    l = pwd + strlen(pwd);
    for(;;) {
        THEBATPWD_GET(a)
        THEBATPWD_GET(d)
        *o++ = (a << 2) | (d >> 4);
        THEBATPWD_GET(a)
        *o++ = (d << 4) | (a >> 2);
        THEBATPWD_GET(d)
        *o++ = (a << 6) | d;
    }

next_step:
    *o = 0;
    l = malloc(o - pwd);
    p = l;
    for(o = pwd + strlen(pwd) - 1; (o >= pwd) && (*o < 0xfe); o--) {
        *p++ = *o ^ 'Z';
    }
    *p = 0;
    strcpy(pwd, l);
    free(l);
}



int main(int argc, char *argv[]) {
    u8      *pwd;

    fputs("\n"
        "TheBat! password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <password>\n"
            "\n"
            "the passwords are located in various places, for example:\n"
            "- usernames and passwords: \"%%APPDATA%%\\The Bat!\\YOUR_ACCOUNT_NAME\\Account.cfn\n"
            "- proxy passwords are located in the registry: \"HKCU\\Software\\RIT\\The Bat!\"\n"
            "- possibly other files inside \"%%APPDATA%%\\The Bat!\"\n"
            "\n", argv[0]);
        exit(1);
    }
    pwd = argv[1];

    thebatpwd(pwd);
    printf("- password: %s\n", pwd);
    return(0);
}

